@@ -14,7 +14,7 @@ Stéphane Bortzmeyer <stephane+frama@bortzmeyer.org>
 """
 
 # WARNING: if you modify it here, also change setup.py https://packaging.python.org/guides/single-sourcing-package-version/#single-sourcing-the-version
-VERSION = '2.1.1'
+VERSION = '2.2'
 
 import os
 import json
@@ -115,11 +115,6 @@ def host_type(s):
         except socket.error: # not a valid IPv4 address either
             return Host_Type.Name
 
-def clean_key (url):
-    """Removes key from URL, for safer error messages."""
-    (url, key) = re.split("\?key=", url)
-    return url
-
 class Config:
     def __init__(self):
         # Default values
@@ -368,12 +363,16 @@ class Config:
         return args, data
     
 class JsonRequest(urllib.request.Request):
-    def __init__(self, url):
+    def __init__(self, url, key):
         urllib.request.Request.__init__(self, url)
         self.url = url
         self.add_header("Content-Type", "application/json")
         self.add_header("Accept", "application/json")
-        self.add_header("User-Agent", "RIPEAtlas.py")
+        self.add_header("User-Agent", "Blaeu/%s" % VERSION)
+        self.add_header("Authorization", "Key %s" % key)
+        #To debug, uncomment this line:
+        #print(self.header_items())
+        #print(self) will display the URL called
     def __str__(self):
         return self.url
 
@@ -411,22 +410,25 @@ class Measurement():
                 key = key.rstrip('\n')
                 auth.close()
 
-        self.url = base_url + "/?key=%s" % key
-        self.url_probes = base_url + "/%s/?fields=probes,status" + "&key=%s" % key
-        self.url_status = base_url + "/%s/?fields=status" + "&key=%s" % key 
-        self.url_results = base_url + "/%s/results/" + "?key=%s" % key
-        self.url_all = base_url + "/%s/" + "?key=%s" % key
+        self.url = base_url 
+        self.url_probes = base_url + "/%s/?fields=probes,status"
+        self.url_status = base_url + "/%s/?fields=status"
+        self.url_results = base_url + "/%s/results/"
+        self.url_all = base_url + "/%s/"
         self.url_latest = base_url + "-latest/%s/?versions=%s"
-
+        self.key = key
         self.status = None
         
         if data is not None:
             self.json_data = json.dumps(data).encode('utf-8')
             self.notification = sleep_notification
-            request = JsonRequest(self.url)
+            request = JsonRequest(self.url, self.key)
             try:
                 # Start the measurement
                 conn = urllib.request.urlopen(request, self.json_data)
+                # To debug, uncomment these two lines:
+                #headers = conn.getheaders()
+                #print(headers)
                 # Now, parse the answer
                 results = json.loads(conn.read().decode('utf-8'))
                 self.id = results["measurements"][0]
@@ -434,7 +436,7 @@ class Measurement():
             except urllib.error.HTTPError as e:
                 raise RequestSubmissionError(e.code, e.reason, e.read().decode())
             except urllib.error.URLError as e:
-                raise RequestSubmissionError(None, e.reason, "URL %s" % clean_key(self.url))
+                raise RequestSubmissionError(None, e.reason, "URL %s" % self.url)
 
             self.gen = random.Random()
             self.time = time.gmtime()
@@ -453,7 +455,7 @@ class Measurement():
                 fields_delay *= 2
                 try:
                     request = JsonRequest((self.url_probes % self.id) + \
-                                          ("&defeatcaching=dc%s" % self.gen.randint(1,10000))) # A random
+                                          ("&defeatcaching=dc%s" % self.gen.randint(1,10000)), self.key) # A random
                                 # component is necesary to defeat caching (even Cache-Control sems ignored)
                     conn = urllib.request.urlopen(request)
                     # Now, parse the answer
@@ -477,7 +479,7 @@ class Measurement():
             self.id = id
             self.notification = None
             try:
-                conn = urllib.request.urlopen(JsonRequest(self.url_status % self.id))
+                conn = urllib.request.urlopen(JsonRequest(self.url_status % self.id, self.key))
             except urllib.error.HTTPError as e:
                 if e.code == 404:
                     raise MeasurementNotFound
@@ -492,7 +494,7 @@ class Measurement():
             if status != "Ongoing" and status != "Stopped":
                 raise MeasurementAccessError("Invalid status \"%s\"" % status)
             try:
-                conn = urllib.request.urlopen(JsonRequest(self.url_probes % self.id))
+                conn = urllib.request.urlopen(JsonRequest(self.url_probes % self.id, self.key))
             except urllib.error.HTTPError as e:
                 if e.code == 404:
                     raise MeasurementNotFound
@@ -504,7 +506,7 @@ class Measurement():
             result_status = json.loads(conn.read().decode('utf-8')) 
             self.num_probes = len(result_status["probes"])
         try:
-                conn = urllib.request.urlopen(JsonRequest(self.url_all % self.id))
+                conn = urllib.request.urlopen(JsonRequest(self.url_all % self.id, self.key))
         except urllib.error.HTTPError as e:
                 if e.code == 404:
                         raise MeasurementNotFound
@@ -532,9 +534,9 @@ class Measurement():
         if latest is not None:
             wait = False
         if latest is None:
-            request = JsonRequest(self.url_results % self.id)
+            request = JsonRequest(self.url_results % self.id, self.key)
         else:
-            request = JsonRequest(self.url_latest% (self.id, latest))
+            request = JsonRequest(self.url_latest % (self.id, latest), self.key)
         if wait:
             enough = False
             attempts = 0
@@ -564,7 +566,7 @@ class Measurement():
                         # have sent only a part of its measurements.
                         enough = True
                     else:
-                        conn = urllib.request.urlopen(JsonRequest(self.url_status % self.id))
+                        conn = urllib.request.urlopen(JsonRequest(self.url_status % self.id, self.key))
                         result_status = json.loads(conn.read().decode('utf-8')) 
                         status = result_status["status"]["name"]
                         if status == "Ongoing":
@@ -619,7 +621,7 @@ class Probe():
             return
         self.url = base_url_probes + "/%s" % id
         try:
-            conn = urllib.request.urlopen(JsonRequest(self.url))
+            conn = urllib.request.urlopen(JsonRequest(self.url, self.key))
         except urllib.error.HTTPError as e:
             if e.code == 404:
                 raise ProbeNotFound
